/* eslint-disable no-magic-numbers */
import {
  Accordion,
  AccordionItem,
  AccordionItemButton,
  AccordionItemHeading,
  AccordionItemPanel,
} from '@/shared/Accordion';
import { useAppSelector } from '@/redux/hooks/useAppSelector';
import {
  elementAnnotationsSelector,
  loadingStatisticsSelector,
} from '@/redux/statistics/statistics.selectors';
import { CheckboxFilter } from '../CheckboxFilter';

export const Annotations = (): React.ReactNode => {
  const loadingStatistics = useAppSelector(loadingStatisticsSelector);
  const elementAnnotations = useAppSelector(elementAnnotationsSelector);
  const isPending = loadingStatistics === 'pending';

  const mappedElementAnnotations = elementAnnotations
    ? Object.keys(elementAnnotations)?.map(el => ({ id: el, label: el }))
    : [];

  return (
    <Accordion allowZeroExpanded>
      <AccordionItem>
        <AccordionItemHeading>
          <AccordionItemButton>Select annotations</AccordionItemButton>
        </AccordionItemHeading>
        <AccordionItemPanel>
          {isPending && <p>Loading...</p>}
          {!isPending && mappedElementAnnotations && mappedElementAnnotations.length > 0 && (
            <CheckboxFilter options={mappedElementAnnotations} />
          )}
        </AccordionItemPanel>
      </AccordionItem>
    </Accordion>
  );
};
