/* eslint-disable no-magic-numbers */
import { CompartmentPathwayDetails, MapModel } from '@/types/models';

type AddedNames = { [key: string]: number };

type CheckboxElement = { id: string; label: string };

type CheckboxElements = CheckboxElement[];

export const getCompartmentPathwaysCheckboxElements = (
  items: CompartmentPathwayDetails[],
): CheckboxElements => {
  const addedNames: AddedNames = {};

  const setNameToIdIfUndefined = (item: CompartmentPathwayDetails): void => {
    if (addedNames[item.name] === undefined) {
      addedNames[item.name] = item.id;
    }
  };

  items.forEach(setNameToIdIfUndefined);

  const parseIdAndLabel = ([name, id]: [name: string, id: number]): CheckboxElement => ({
    id: id.toString(),
    label: name,
  });

  const sortByLabel = (a: CheckboxElement, b: CheckboxElement): number => {
    if (a.label > b.label) return 1;
    return -1;
  };

  const elements = Object.entries(addedNames).map(parseIdAndLabel).sort(sortByLabel);

  return elements;
};

export const getModelsIds = (models: MapModel[] | undefined): number[] => {
  if (!models) return [];

  return models.map(model => model.idObject);
};
