/* eslint-disable no-magic-numbers */
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { mockNetworkResponse } from '@/utils/mockNetworkResponse';
import { HttpStatusCode } from 'axios';
import {
  compartmentPathwaysDetailsFixture,
  compartmentPathwaysFixture,
  compartmentPathwaysOverLimitFixture,
} from '@/models/fixtures/compartmentPathways';
import { getModelsIds } from '@/components/Map/Drawer/ExportDrawer/Elements/Elements.utils';
import { apiPath } from '../apiPath';
import compartmentPathwaysReducer from './compartmentPathways.slice';
import { CompartmentPathwaysState } from './compartmentPathways.types';
import { getCompartmentPathways } from './compartmentPathways.thunks';
import { MODELS_MOCK, MODELS_MOCK_SHORT } from './compartmentPathways.mock';

const mockedAxiosClient = mockNetworkResponse();
const MODELS_MOCK_IDS = getModelsIds(MODELS_MOCK);

describe('compartmentPathways thunk', () => {
  let store = {} as ToolkitStoreWithSingleSlice<CompartmentPathwaysState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('compartmentPathways', compartmentPathwaysReducer);
  });

  it('should handle query getCompartmentPathways properly when models are undefined', async () => {
    mockedAxiosClient
      .onGet(apiPath.getCompartmentPathwaysIds(52))
      .reply(HttpStatusCode.Ok, compartmentPathwaysFixture);
    mockedAxiosClient
      .onGet(apiPath.getCompartmentPathwayDetails([1, 2, 3]))
      .reply(HttpStatusCode.Ok, compartmentPathwaysDetailsFixture);
    mockedAxiosClient
      .onPost(apiPath.sendCompartmentPathwaysIds())
      .reply(HttpStatusCode.Ok, compartmentPathwaysDetailsFixture);

    const { loading, data } = store.getState().compartmentPathways;

    expect(loading).toEqual('idle');
    expect(data).toEqual([]);

    const comparmentPathwaysPromise = store.dispatch(getCompartmentPathways());

    const { loading: loadingPending, data: dataPending } = store.getState().compartmentPathways;

    expect(loadingPending).toEqual('pending');
    expect(dataPending).toEqual([]);

    await comparmentPathwaysPromise;
    const { loading: loadingFulfilled, data: dataFulfilled } = store.getState().compartmentPathways;

    expect(loadingFulfilled).toEqual('succeeded');
    expect(dataFulfilled).toEqual([]);
  });
  it('should handle sendCompartmentPathwaysIds request properly if it is more than 100 ids', async () => {
    mockedAxiosClient
      .onGet(apiPath.getCompartmentPathwaysIds(5053))
      .reply(HttpStatusCode.Ok, compartmentPathwaysFixture);
    mockedAxiosClient
      .onGet(apiPath.getCompartmentPathwaysIds(5054))
      .reply(HttpStatusCode.Ok, compartmentPathwaysOverLimitFixture);

    const ids = compartmentPathwaysFixture.map(el => el.id);
    mockedAxiosClient
      .onGet(apiPath.getCompartmentPathwayDetails(ids))
      .reply(HttpStatusCode.Ok, compartmentPathwaysDetailsFixture);
    mockedAxiosClient
      .onPost(apiPath.sendCompartmentPathwaysIds())
      .reply(HttpStatusCode.Ok, compartmentPathwaysDetailsFixture);

    const compartmentPathwaysPromise = store.dispatch(getCompartmentPathways(MODELS_MOCK_IDS));

    const { loading, data } = store.getState().compartmentPathways;

    expect(loading).toEqual('pending');
    expect(data).toEqual([]);

    const { type } = await compartmentPathwaysPromise;

    expect(type).toBe('compartmentPathways/getCompartmentPathways/fulfilled');

    const { loading: promiseFulfilled, data: dataFulfilled } = store.getState().compartmentPathways;

    expect(dataFulfilled).toEqual([
      ...compartmentPathwaysDetailsFixture,
      ...compartmentPathwaysDetailsFixture,
    ]);
    expect(promiseFulfilled).toEqual('succeeded');
  });

  it('should not do a network request sendCompartmentPathwaysIds if it is less than 100 ids', async () => {
    const ONE_MODEL = MODELS_MOCK_SHORT[0];
    const ID = ONE_MODEL.idObject;

    mockedAxiosClient
      .onGet(apiPath.getCompartmentPathwaysIds(ID))
      .reply(HttpStatusCode.Ok, compartmentPathwaysFixture);

    const ids = compartmentPathwaysFixture.map(el => el.id);
    mockedAxiosClient
      .onGet(apiPath.getCompartmentPathwayDetails(ids))
      .reply(HttpStatusCode.Ok, compartmentPathwaysDetailsFixture);
    mockedAxiosClient
      .onPost(apiPath.sendCompartmentPathwaysIds())
      .reply(HttpStatusCode.Ok, compartmentPathwaysDetailsFixture);

    const compartmentPathwaysPromise = store.dispatch(getCompartmentPathways([ONE_MODEL.idObject]));

    const { loading, data } = store.getState().compartmentPathways;

    expect(loading).toEqual('pending');
    expect(data).toEqual([]);

    const { type } = await compartmentPathwaysPromise;

    expect(type).toBe('compartmentPathways/getCompartmentPathways/fulfilled');

    const { loading: promiseFulfilled, data: dataFulfilled } = store.getState().compartmentPathways;

    expect(dataFulfilled).toEqual(compartmentPathwaysDetailsFixture);
    expect(promiseFulfilled).toEqual('succeeded');
  });
});
